#include <libusb.h>
#include <string.h>
#include "usbctrl.h"

/*----------------------------------------------------------------------------*/
int control_request( libusb_device_handle* hdev, 
							int type, int request, int value, int index,
							char* buf, int sz, int timeout 
						)
{
/*----------------------------------------------------------------------------*/
#ifdef LIBUSB_1_0
	return libusb_control_transfer( hdev, 
		    type, request, value, index, (unsigned char*)buf, sz, timeout );
#else
	return usb_control_msg( hdev, 
						type, request, value, index, buf, sz, timeout );
#endif
}
/*----------------------------------------------------------------------------*/
int sub_get_serial_number( libusb_device_handle* hdev, char *buf, int sz)
{
#ifdef LIBUSB_1_0
	return libusb_get_string_descriptor_ascii(hdev, 3, (unsigned char*)buf, sz);
#else
	return usb_get_string_simple(hdev, 3, buf, sz);
#endif
}
/*----------------------------------------------------------------------------*/
int sub_get_product_id( libusb_device_handle* hdev, char *buf, int sz)
{
#ifdef LIBUSB_1_0
	return libusb_get_string_descriptor_ascii(hdev, 2, (unsigned char*)buf, sz);
#else
	return usb_get_string_simple(hdev, 2, buf, sz);
#endif
}
/*----------------------------------------------------------------------------*/
const struct sub_version* sub_get_version( libusb_device_handle* hdev )
{
	uint8_t		rc;
	static struct sub_version sub_ver;

	struct get_version_resp_st ver;

	memset( &sub_ver.sub_device, 0xFF, sizeof(sub_ver.sub_device) ) ;

	rc=control_request(
		hdev, 
		CTRL_REQ_TRANSFER_DIR_IN|CTRL_REQ_TYPE_VENDOR|CTRL_REQ_RCPT_DEVICE,
		SUB_VENDOR_REQUEST,
		0,
		SUB_WINDEX_VER,
		(char*)&ver, 
		sizeof(ver), 
		10000 );

	if( rc == sizeof(ver) )
	{
		sub_ver.sub_device.micro = ver.micro;
		sub_ver.sub_device.minor = ver.minor;
		sub_ver.sub_device.major = ver.major;
		sub_ver.sub_device.boot  = ver.boot;
	}

	return &sub_ver;
}
/*----------------------------------------------------------------------------*/
const sub_vpd_t* sub_get_cfg_vpd( libusb_device_handle* hdev )
{
	uint8_t		rc;
	static sub_vpd_t cfg_vpd;
	sub_vpd_t	vpd;

	rc=control_request(
		hdev, 
		CTRL_REQ_TRANSFER_DIR_IN|CTRL_REQ_TYPE_VENDOR|CTRL_REQ_RCPT_DEVICE,
		SUB_VENDOR_REQUEST,
		0,
		SUB_WINDEX_CFG_VPD,
		(char*)&vpd, 
		sizeof(vpd), 
		10000 );

	if( rc == sizeof(vpd) )
	{
		cfg_vpd.lcd_w 	= vpd.lcd_w;
		cfg_vpd.lcd_h 	= vpd.lcd_h;
		cfg_vpd.serial	= vpd.serial;
		cfg_vpd.buttons	= vpd.buttons;
		cfg_vpd.ir_car	= vpd.ir_car;
	}
	else
		return 0;

	return &cfg_vpd;
}
/*----------------------------------------------------------------------------*/

int sub_update_nvram( libusb_device_handle* hdev, char *buf, int size )
{
	int rc;
	rc = control_request(
		hdev, 
		CTRL_REQ_TRANSFER_DIR_OUT|CTRL_REQ_TYPE_VENDOR|CTRL_REQ_RCPT_DEVICE,
		SUB_BOOT_WRITE_NVRAM,
		0,
		0,
		buf, 
		size, 
		10000 );
	return rc==size?0:-1;
}
/*----------------------------------------------------------------------------*/
int sub_get_mode( libusb_device_handle* hdev, int *mode )
{
	int bc;
	uint8_t m;
	bc=control_request(
		hdev, 
		CTRL_REQ_TRANSFER_DIR_IN|CTRL_REQ_TYPE_VENDOR|CTRL_REQ_RCPT_DEVICE,
		SUB_VENDOR_REQUEST,
		0,
		SUB_WINDEX_MODE,
		(char*)&m, 
		sizeof(m), 
		10000 );

	if( bc==1 )
	{
		*mode=m;
	}
	return bc>0?0:bc;
}
/*----------------------------------------------------------------------------*/
int sub_set_mode( libusb_device_handle* hdev, int mode )
{
	int rc = control_request( 
		hdev, 
		CTRL_REQ_TYPE_VENDOR|CTRL_REQ_RCPT_DEVICE,
		SUB_VENDOR_REQUEST, 
		0, 
		0, 
		0, 
		0, 
		1000 );

	return rc<0?-1:0;
}
/*----------------------------------------------------------------------------*/
