#ifndef __USBCTRL_H__
#define __USBCTRL_H__


/* USB vendor requests definitions */
#define CTRL_REQ_TRANSFER_DIR_IN		(1<<7)
#define CTRL_REQ_TRANSFER_DIR_OUT		0

#define CTRL_REQ_TYPE_STANDARD			(0<<5)
#define CTRL_REQ_TYPE_CLASS				(1<<5)
#define CTRL_REQ_TYPE_VENDOR			(2<<5)

#define CTRL_REQ_RCPT_DEVICE			0
#define CTRL_REQ_RCPT_INTERFACE			1
#define CTRL_REQ_RCPT_ENDPOINT			2
#define CTRL_REQ_RCPT_OTHER				3

/* Vendor Specific Control Request wIndex */
#define SUB_VENDOR_REQUEST		0x80
#define SUB_WINDEX_BOOT			0x00
#define SUB_WINDEX_MODE			0x01

#define SUB_WINDEX_VER			0x02
#define SUB_WINDEX_CFG_VPD		0x03

/* Vendor requests in boot mode */
#define SUB_BOOT_WRITE_NVRAM	1

#define SUB_MAX_PACK		64
#define SUB_MAX_PACK_LONG	256

#define SUB_BOOT_MODE	0



struct sub_version
{
	struct 
	{
		int major;
		int minor;
		int micro;
		int nano;
	}dll;
	
	struct 
	{
		int major;
		int minor;
		int micro;
		int nano;
	}driver;

	struct
	{
		int micro;
		int minor;
		int major;
		int boot;
	}sub_device;
};

typedef struct
{
    uint8_t lcd_w;      /* LCD With (0-no LCD, 8, 16) */
    uint8_t lcd_h;      /* LCD Lines 1,2,... */
    uint8_t serial;     /* 2-RS232, 4-RS485 */
    uint8_t buttons;    /* 0,1,2 */
    uint16_t ir_car;    /* 0-no IR, IR carrier frequency Hz */
}sub_vpd_t;


struct get_version_resp_st
{
	uint8_t		micro;
	uint8_t		minor;
	uint8_t		major;
	uint8_t		boot;
}get_version_resp;



int sub_get_serial_number( libusb_device_handle* hdev, char *buf, int sz);
int sub_get_product_id( libusb_device_handle* hdev, char *buf, int sz);
const struct sub_version* sub_get_version( libusb_device_handle* hdev );
const sub_vpd_t* sub_get_cfg_vpd( libusb_device_handle* hdev );
int sub_update_nvram( libusb_device_handle* hdev, char *buf, int size );
int sub_get_mode( libusb_device_handle* hdev, int *mode );
int sub_set_mode( libusb_device_handle* hdev, int mode );



#endif	/*__USBCTRL_H__*/
